{-# LANGUAGE DeriveDataTypeable #-}
--------------------------------------------------------------------------
-- |
-- Module      :  Main
-- Copyright   :  (c) Thanos Tsouanas 2010
-- License     :  BSD
-- Maintainer  :  thanos@sians.org
-- Stability   :  experimental
--
-- TsouTAPL: implementations of various lambda calculus based systems.
--
--------------------------------------------------------------------------
module Main ( main ) where
--------------------------------------------------------------------------
import Untyped
import Commander
import System.Console.CmdArgs    ( isQuiet, isNormal, isLoud )
--------------------------------------------------------------------------


-- | Gets the command-line arguments and runs in the appropriate mode.
main :: IO ()
main = do tsoutapl <- processArgs
          case tsoutapl of
               opt@Untyped {} -> runUntyped opt
               opt@Typed   {} -> runTyped opt


{- Different modes -}

-- Untyped lambda calculus with unit, booleans and arithmetic.
runUntyped :: TsouTAPL -> IO ()
runUntyped opt = do prgFile <- readFile $ file opt
                    quiet   <- isQuiet
                    verbose <- isLoud
                    normal  <- isNormal
                    let interpreter | verbose          = interpret
                                    | normal           = biginterpret
                                    | quiet            = quietinterpret
                    let computer    | unprotected opt  = compute
                                    | otherwise        = safecompute
                    putStrLn $ interpreter computer prgFile


-- Simply typed lambda calculus with unit, booleans, arithmetic and wrong.
runTyped :: TsouTAPL -> IO ()
runTyped opt  = do putStrLn  "The typed system has not been implemented in\
                             \ version 2.* yet.\n Check the latest 1.* version\
                             \ if you need it."
